@@ -31,9 +31,20 @@ import platform
 import sys
 from distutils.log import info
 
+try:
+    import distro
+except ImportError:
+    python_version = tuple(int(x) for x in platform.python_version_tuple()[:2])
+    if python_version >= (3, 8):
+        print(
+            'Missing build requirement "distro" Python module; '
+            'install paths may be incorrect', file=sys.stderr)
+
+windows_build = os.name == 'nt'
+
 
 def has_help(self):
-    return "build_help" in self.distribution.cmdclass and os.name != 'nt'
+    return "build_help" in self.distribution.cmdclass and not windows_build
 
 
 def has_icons(self):
@@ -41,7 +52,7 @@ def has_icons(self):
 
 
 def has_i18n(self):
-    return "build_i18n" in self.distribution.cmdclass and os.name != 'nt'
+    return "build_i18n" in self.distribution.cmdclass and not windows_build
 
 
 def has_data(self):
@@ -78,6 +89,14 @@ class build_data(distutils.cmd.Command):
         ('share/meld', ['data/gschemas.compiled']),
     ]
 
+    style_source = "data/styles/*.style-scheme.xml.in"
+    style_target_dir = 'share/meld/styles'
+
+    # FIXME: This is way too much hard coding, but I really hope
+    # it also doesn't last that long.
+    resource_source = "meld/resources/meld.gresource.xml"
+    resource_target = "org.gnome.Meld.gresource"
+
     def initialize_options(self):
         pass
 
@@ -85,10 +104,46 @@ class build_data(distutils.cmd.Command):
         pass
 
     def get_data_files(self):
-        if os.name == 'nt':
-            return self.frozen_gschemas
+        data_files = []
+
+        build_path = os.path.join('build', 'data')
+        if not os.path.exists(build_path):
+            os.makedirs(build_path)
+
+        info("compiling gresources")
+        resource_dir = os.path.dirname(self.resource_source)
+        target = os.path.join(build_path, self.resource_target)
+        self.spawn([
+            "glib-compile-resources",
+            "--target={}".format(target),
+            "--sourcedir={}".format(resource_dir),
+            self.resource_source,
+        ])
+
+        data_files.append(('share/meld', [target]))
+
+        if windows_build:
+            gschemas = self.frozen_gschemas
         else:
-            return self.gschemas
+            gschemas = self.gschemas
+        data_files.extend(gschemas)
+
+        if windows_build:
+            # These should get moved/installed by i18n, but until that
+            # runs on Windows we need this hack.
+            styles = glob.glob(self.style_source)
+
+            import shutil
+            targets = []
+            for style in styles:
+                assert style.endswith('.in')
+                target = style[:-len('.in')]
+                shutil.copyfile(style, target)
+                targets.append(target)
+
+            data_files.append((self.style_target_dir, targets))
+
+        return data_files
 
     def run(self):
         data_files = self.distribution.data_files
@@ -196,7 +251,7 @@ class build_icons(distutils.cmd.Command):
         pass
 
     def run(self):
-        target_dir = self.frozen_target if os.name == 'nt' else self.target
+        target_dir = self.frozen_target if windows_build else self.target
         data_files = self.distribution.data_files
 
         for theme in glob.glob(os.path.join(self.icon_dir, "*")):
@@ -227,6 +282,7 @@ class build_i18n(distutils.cmd.Command):
     # way except magically extracting them from self.distribution.data_files
     desktop_files = [('share/applications', glob.glob("data/*.desktop.in"))]
     xml_files = [
+        ('share/meld/styles', glob.glob("data/styles/*.style-scheme.xml.in")),
         ('share/metainfo', glob.glob("data/*.appdata.xml.in")),
         ('share/mime/packages', glob.glob("data/mime/*.xml.in"))
     ]
@@ -251,7 +307,7 @@ class build_i18n(distutils.cmd.Command):
 
         # If we're on Windows, assume we're building frozen and make a bunch
         # of insane assumptions.
-        if os.name == 'nt':
+        if windows_build:
             msgfmt = "C:\\Python27\\Tools\\i18n\\msgfmt"
         else:
             msgfmt = "msgfmt"
@@ -293,6 +349,18 @@ class build_i18n(distutils.cmd.Command):
             os.environ["XGETTEXT_ARGS"] = "--msgid-bugs-address=%s " % \
                                           self.bug_contact
 
+        # These copies are pure hacks to work around not having the
+        # Meson-based initial variable templating in distutils.
+        import shutil
+        shutil.copyfile(
+            'data/org.gnome.meld.desktop.in.in',
+            'data/org.gnome.meld.desktop.in',
+        )
+        shutil.copyfile(
+            'data/org.gnome.meld.appdata.xml.in.in',
+            'data/org.gnome.meld.appdata.xml.in',
+        )
+
         self._rebuild_po()
 
         intltool_switches = [
@@ -378,11 +446,21 @@ class install(distutils.command.install.install):
 
     def finalize_options(self):
         special_cases = ('debian', 'ubuntu', 'linuxmint')
-        if (platform.system() == 'Linux' and
-                platform.linux_distribution()[0].lower() in special_cases):
-            # Maintain an explicit install-layout, but use deb by default
-            specified_layout = getattr(self, 'install_layout', None)
-            self.install_layout = specified_layout or 'deb'
+        if platform.system() == 'Linux':
+            # linux_distribution has been removed in Python 3.8; we require
+            # the third-party distro package for future handling.
+            try:
+                distribution = platform.linux_distribution()[0].lower()
+            except AttributeError:
+                try:
+                    distribution = distro.id()
+                except NameError:
+                    distribution = 'unknown'
+
+            if distribution in special_cases:
+                # Maintain an explicit install-layout, but use deb by default
+                specified_layout = getattr(self, 'install_layout', None)
+                self.install_layout = specified_layout or 'deb'
 
         distutils.command.install.install.finalize_options(self)
 
