@@ -1,22 +1,35 @@
 
 import os
 import sys
+from pathlib import Path
 
 __package__ = "meld"
-__version__ = "3.20.1"
+__version__ = "3.21.0"
 
-APPLICATION_ID = "org.gnome.meld"
+APPLICATION_NAME = 'Meld'
+APPLICATION_ID = 'org.gnome.Meld'
+SETTINGS_SCHEMA_ID = 'org.gnome.meld'
+RESOURCE_BASE = '/org/gnome/meld'
 
 DATADIR = "/usr/local/share/meld"
 LOCALEDIR = "/usr/local/share/locale"
 
+CONFIGURED = '@configured@'
+PROFILE = ''
+
+if CONFIGURED == 'True':
+    APPLICATION_ID = '@application_id@'
+    DATADIR = '@pkgdatadir@'
+    LOCALEDIR = '@localedir@'
+    PROFILE = '@profile@'
+
 # Flag enabling some workarounds if data dir isn't installed in standard prefix
 DATADIR_IS_UNINSTALLED = False
-PYTHON_REQUIREMENT_TUPLE = (3, 3)
+PYTHON_REQUIREMENT_TUPLE = (3, 6)
 
 
 # Installed from main script
-def no_translation(gettext_string, *args):
+def no_translation(gettext_string: str) -> str:
     return gettext_string
 
 
@@ -36,13 +49,12 @@ def frozen():
 
 def uninstalled():
     global DATADIR, LOCALEDIR, DATADIR_IS_UNINSTALLED
-    melddir = os.path.abspath(os.path.join(
-        os.path.dirname(os.path.realpath(__file__)), ".."))
 
-    DATADIR = os.path.join(melddir, "data")
-    LOCALEDIR = os.path.join(melddir, "build", "mo")
+    melddir = Path(__file__).resolve().parent.parent
+
+    DATADIR = melddir / "data"
+    LOCALEDIR = melddir / "build" / "mo"
     DATADIR_IS_UNINSTALLED = True
 
-
-def ui_file(filename):
-    return os.path.join(DATADIR, "ui", filename)
\ No newline at end of file
+    resource_path = melddir / "meld" / "resources"
+    os.environ['G_RESOURCE_OVERLAYS'] = f'{RESOURCE_BASE}={resource_path}'
\ No newline at end of file
