@@ -3,8 +3,9 @@
 # module, allowing editing of our UI files without having widgets that
 # can't be instantiated everywhere.
 
+from meld import actiongutter  # noqa: F401
+from meld import chunkmap  # noqa: F401
 from meld import diffgrid  # noqa: F401
-from meld import diffmap  # noqa: F401
 from meld import linkmap  # noqa: F401
 from meld import preferences  # noqa: F401
 from meld import sourceview  # noqa: F401
@@ -12,4 +13,5 @@ from meld.ui import emblemcellrenderer  # noqa: F401
 from meld.ui import historyentry  # noqa: F401
 from meld.ui import msgarea  # noqa: F401
 from meld.ui import notebook  # noqa: F401
+from meld.ui import recentselector  # noqa: F401
 from meld.ui import statusbar  # noqa: F401
